@@ -31,6 +31,7 @@
 #include "core/proto.h"
 #include "core/processing.h"
 #include "io/single_image.h"
+#include "algos/statistics.h"
 #include "algos/fft.h"
 
 void fft_to_spectra(fits* fit, fftw_complex *frequency_repr, double *as,
@@ -207,6 +208,7 @@ void FFTI(fits *fit, fits *xfit, fits *yfit, int type_order, int layer) {
 		gbuf[i] = round_to_WORD(pxl);
 	}
 	delete_selected_area();
+	invalidate_stats_from_fit(fit);
 
 	free(modulus);
 	free(phase);
@@ -285,14 +287,14 @@ gpointer fourier_transform(gpointer p) {
 		tmp = calloc(1, sizeof(fits));
 		if (readfits(args->modulus, tmp, NULL)) {
 			free(tmp);
-			gdk_threads_add_idle(end_fourier_transform, args);
+			siril_add_idle(end_fourier_transform, args);
 			return GINT_TO_POINTER(1);
 		}
 		tmp1 = calloc(1, sizeof(fits));
 		if (readfits(args->phase, tmp1, NULL)) {
 			free(tmp);
 			free(tmp1);
-			gdk_threads_add_idle(end_fourier_transform, args);
+			siril_add_idle(end_fourier_transform, args);
 			return GINT_TO_POINTER(1);
 		}
 		if (tmp->dft_ord[0] == 'C')		// CENTERED
@@ -303,7 +305,7 @@ gpointer fourier_transform(gpointer p) {
 			free(tmp);
 			free(tmp1);
 			siril_log_message(_("There is something wrong in your files\n"));
-			gdk_threads_add_idle(end_fourier_transform, args);
+			siril_add_idle(end_fourier_transform, args);
 			return GINT_TO_POINTER(1);
 		}
 		new_fit_image(&tmp2, tmp->rx, tmp->ry, tmp->naxes[2]);
@@ -318,13 +320,17 @@ gpointer fourier_transform(gpointer p) {
 					tmp->dft_rx * tmp->dft_ry * sizeof(WORD));
 		}
 	}
+	invalidate_stats_from_fit(args->fit);
 	clearfits(tmp);
 	clearfits(tmp1);
 	clearfits(tmp2);
+	free(tmp);
+	free(tmp1);
+	free(tmp2);
 
 	gettimeofday(&t_end, NULL);
 	show_time(t_start, t_end);
-	gdk_threads_add_idle(end_fourier_transform, args);
+	siril_add_idle(end_fourier_transform, args);
 
 	return GINT_TO_POINTER(0);
 }
