@@ -1,16 +1,9 @@
 #ifndef FINDER_H_
 #define FINDER_H_
 
-typedef struct starFinder_struct starFinder;
-
-struct starFinder_struct {
-	int radius;
-	double sigma;
-	double roundness;
-	int nb_stars;
-};
-
-fitted_PSF **peaker(fits *fit, int layer, starFinder *sf, rectangle *area);
+void init_peaker_GUI();
+void init_peaker_default();
+fitted_PSF **peaker(fits *fit, int layer, star_finder_params *sf, int *nb_stars, rectangle *area, gboolean showtime);
 fitted_PSF *add_star(fits *fit, int layer, int *index);
 int remove_star(int index);
 void sort_stars(fitted_PSF **stars, int total);
