@@ -48,6 +48,7 @@
 #include "gui/histogram.h"
 #include "gui/progress_and_log.h"
 #include "gui/PSF_list.h"
+#include "io/conversion.h"
 #include "io/sequence.h"
 #include "io/single_image.h"
 #include "io/ser.h"
@@ -56,6 +57,7 @@
 #include "algos/Def_Math.h"
 #include "algos/Def_Wavelet.h"
 #include "algos/cosmetic_correction.h"
+#include "algos/statistics.h"
 #include "opencv/opencv.h"
 
 #define MAX_ITER 15
@@ -73,6 +75,7 @@ int threshlo(fits *fit, int level) {
 			buf++;
 		}
 	}
+	invalidate_stats_from_fit(fit);
 	return 0;
 }
 
@@ -86,6 +89,7 @@ int threshhi(fits *fit, int level) {
 			buf++;
 		}
 	}
+	invalidate_stats_from_fit(fit);
 	return 0;
 }
 
@@ -100,6 +104,7 @@ int nozero(fits *fit, int level) {
 			buf++;
 		}
 	}
+	invalidate_stats_from_fit(fit);
 	return 0;
 }
 
@@ -142,6 +147,7 @@ int soper(fits *a, double scalar, char oper) {
 			break;
 		}
 	}
+	invalidate_stats_from_fit(a);
 	return 0;
 }
 
@@ -179,11 +185,12 @@ int imoper(fits *a, fits *b, char oper) {
 			break;
 		case OPER_DIV:
 			for (i = 0; i < n; ++i) {
-				gbuf[i] = round_to_WORD(gbuf[i] / buf[i]);
+				gbuf[i] = (buf[i] == 0) ? 0 : round_to_WORD(gbuf[i] / buf[i]);
 			}
 			break;
 		}
 	}
+	invalidate_stats_from_fit(a);
 	return 0;
 }
 
@@ -201,7 +208,7 @@ int sub_background(fits* image, fits* background, int layer) {
 		char *msg = siril_log_message(
 				_("Images don't have the same size (w = %d|%d, h = %d|%d)\n"),
 				image->rx, background->rx, image->ry, background->ry);
-		show_dialog(msg, _("Error"), "dialog-error");
+		show_dialog(msg, _("Error"), "dialog-error-symbolic");
 		return 1;
 	}
 	ndata = image->rx * image->ry;
@@ -209,8 +216,8 @@ int sub_background(fits* image, fits* background, int layer) {
 	/* First step we convert data, apply the subtraction, normalize with median,
 	 * and re-convert data to USHORT
 	 */
-	imstats *stat = statistics(image, layer, NULL, STATS_BASIC, STATS_ZERO_NULLCHECK);
-	median = (double) stat->median / USHRT_MAX_DOUBLE;
+	imstats *stat = statistics(NULL, -1, image, layer, NULL, STATS_BASIC);
+	median = stat->median / USHRT_MAX_DOUBLE;
 	pxl_image = malloc(sizeof(double) * ndata);
 	pxl_bkg = malloc(sizeof(double) * ndata);
 
@@ -222,8 +229,9 @@ int sub_background(fits* image, fits* background, int layer) {
 		image_buf[i] = round_to_WORD(pxl_image[i] * USHRT_MAX_DOUBLE);
 	}
 
+	invalidate_stats_from_fit(image);
 	// We free memory
-	free(stat);
+	free_stats(stat);
 	free(pxl_image);
 	free(pxl_bkg);
 	return 0;
@@ -248,6 +256,7 @@ int addmax(fits *a, fits *b) {
 				gbuf[layer][i] = buf[layer][i];
 		}
 	}
+	invalidate_stats_from_fit(a);
 	return 0;
 }
 
@@ -274,6 +283,7 @@ int fdiv(fits *a, fits *b, float coef) {
 			gbuf[i] = round_to_WORD(temp);
 		}
 	}
+	invalidate_stats_from_fit(a);
 	return retvalue;
 }
 
@@ -301,12 +311,13 @@ int ndiv(fits *a, fits *b) {
 						/ (double) b->pdata[layer][i];
 			max = max(div[i], max);
 		}
-		norm = max / (double) a->max[layer];
+		norm = max / fit_get_max(a, layer);
 		for (i = 0; i < nb_pixels; ++i) {
 			a->pdata[layer][i] = round_to_WORD(div[i] / norm);
 		}
 	}
 
+	invalidate_stats_from_fit(a);
 	free(div);
 	return 0;
 }
@@ -374,6 +385,7 @@ int crop(fits *fit, rectangle *bounds) {
 		gettimeofday(&t_end, NULL);
 		show_time(t_start, t_end);
 	}
+	invalidate_stats_from_fit(fit);
 
 	return 0;
 }
@@ -410,6 +422,7 @@ int shift(int sx, int sy) {
 				gfit.rx * gfit.ry * sizeof(WORD));
 	}
 	free(tmpfit.data);
+	invalidate_stats_from_fit(&gfit);
 
 	return 0;
 }
@@ -465,35 +478,21 @@ int loglut(fits *fit, int dir) {
 				buf[layer][i] = exp(temp / normalisation);
 		}
 	}
+	invalidate_stats_from_fit(fit);
 	return 0;
 }
 
-double contrast(fits* fit, int layer) {
-	int i;
-	WORD *buf = fit->pdata[layer];
-	double contrast = 0.0;
-	imstats *stat = statistics(fit, layer, &com.selection, STATS_BASIC, STATS_ZERO_NULLCHECK);
-	if (!stat) {
-		siril_log_message(_("Error: no data computed.\n"));
-		return -1.0;
-	}
-	double mean = stat->mean;
-	free(stat);
-
-	for (i = 0; i < fit->rx * fit->ry; i++)
-		contrast += SQR((double )buf[i] - mean);
-	contrast /= (fit->rx * fit->ry);
-	return contrast;
-}
-
 int ddp(fits *a, int level, float coeff, float sigma) {
-	copyfits(a, &wfit[0], CP_ALLOC | CP_COPYA | CP_FORMAT, 0);
-	unsharp(&wfit[0], sigma, 0, FALSE);
-	soper(&wfit[0], (double) level, OPER_ADD);
-	nozero(&wfit[0], 1);
-	fdiv(a, &wfit[0], level);
+	fits fit;
+	memset(&fit, 0, sizeof(fits));
+	copyfits(a, &fit, CP_ALLOC | CP_COPYA | CP_FORMAT, 0);
+	unsharp(&fit, sigma, 0, FALSE);
+	soper(&fit, (double) level, OPER_ADD);
+	nozero(&fit, 1);
+	fdiv(a, &fit, level);
 	soper(a, (double) coeff, OPER_MUL);
-	clearfits(&wfit[0]);
+	clearfits(&fit);
+	invalidate_stats_from_fit(a);
 	return 0;
 }
 
@@ -543,6 +542,7 @@ int fill(fits *fit, int level, rectangle *arearg) {
 			buf += stridebuf;
 		}
 	}
+	invalidate_stats_from_fit(fit);
 	return 0;
 }
 
@@ -568,6 +568,7 @@ int off(fits *fit, int level) {
 				buf[layer][i] = val + level;
 		}
 	}
+	invalidate_stats_from_fit(fit);
 	return 0;
 }
 
@@ -680,11 +681,12 @@ int lrgb(fits *l, fits *r, fits *g, fits *b, fits *lrgb) {
 	//
 	// some stats used to normalize
 	//
-	image_find_minmax(r, 0);
-	image_find_minmax(g, 0);
-	image_find_minmax(b, 0);
+	if (image_find_minmax(r) || image_find_minmax(g) ||
+			image_find_minmax(b) || image_find_minmax(l)) {
+		siril_log_color_message(_("Could not compute normalization values for the images, aborting.\n"), "red");
+		return -1;
+	}
 	maxi = max(r->maxi, max(g->maxi, b->maxi));
-	image_find_minmax(l, 0);
 	//
 	// initialize pointers
 	//
@@ -739,7 +741,7 @@ int lrgb(fits *l, fits *r, fits *g, fits *b, fits *lrgb) {
 static double evaluateNoiseOfCalibratedImage(fits *fit, fits *dark, double k) {
 	double noise = 0;
 	fits *dark_tmp = NULL, *fit_tmp = NULL;
-	int chan;
+	int chan, ret = 0;
 
 	if (new_fit_image(&dark_tmp, dark->rx, dark->ry, 1)) {
 		return -1.0;
@@ -753,17 +755,22 @@ static double evaluateNoiseOfCalibratedImage(fits *fit, fits *dark, double k) {
 	copyfits(fit, fit_tmp, CP_ALLOC | CP_EXTRACT, 0);
 
 	soper(dark_tmp, k, OPER_MUL);
-	imoper(fit_tmp, dark_tmp, OPER_SUB);
+	ret = imoper(fit_tmp, dark_tmp, OPER_SUB);
+	if (ret) {
+		clearfits(dark_tmp);
+		clearfits(fit_tmp);
+		return -1.0;
+	}
 
 	for (chan = 0; chan < fit->naxes[2]; chan++) {
-		imstats *stat = statistics(fit_tmp, chan, NULL, STATS_BASIC, STATS_ZERO_NULLCHECK);
+		imstats *stat = statistics(NULL, -1, fit_tmp, chan, NULL, STATS_NOISE);
 		if (!stat) {
-			siril_log_message(_("Error: no data computed.\n"));
+			siril_log_message(_("Error: statistics computation failed.\n"));
 			return 0.0;
 		}
 		noise += stat->bgnoise;
 		//printf("noise=%lf, k=%lf\n", noise, k);
-		free(stat);
+		free_stats(stat);
 	}
 	clearfits(dark_tmp);
 	clearfits(fit_tmp);
@@ -777,10 +784,12 @@ static double goldenSectionSearch(fits *brut, fits *dark, double a, double b,
 		double tol) {
 	double c, d;
 	double fc, fd;
+	int iter = 0;
 
 	c = b - GR * (b - a);
 	d = a + GR * (b - a);
 	do {
+		fprintf(stdout, "Iter: %d\n", ++iter);
 		fc = evaluateNoiseOfCalibratedImage(brut, dark, c);
 		fd = evaluateNoiseOfCalibratedImage(brut, dark, d);
 		if (fc < 0.0 || fd < 0.0)
@@ -799,14 +808,19 @@ static double goldenSectionSearch(fits *brut, fits *dark, double a, double b,
 }
 
 static int preprocess(fits *brut, fits *offset, fits *dark, fits *flat, float level) {
+	int ret = 0;
 
 	if (com.preprostatus & USE_OFFSET) {
-		imoper(brut, offset, OPER_SUB);
+		ret = imoper(brut, offset, OPER_SUB);
+		if (ret)
+			return ret;
 	}
 
 	/* if dark optimization, the master-dark has already been subtracted */
 	if ((com.preprostatus & USE_DARK) && !(com.preprostatus & USE_OPTD)) {
-		imoper(brut, dark, OPER_SUB);
+		ret = imoper(brut, dark, OPER_SUB);
+		if (ret)
+			return ret;
 	}
 
 	if (com.preprostatus & USE_FLAT) {
@@ -820,13 +834,20 @@ static int darkOptimization(fits *brut, fits *dark, fits *offset) {
 	double k;
 	double lo = 0.0;
 	double up = 2.0;
+	int ret = 0;
 	fits *dark_tmp = NULL;
 
+	if (brut->rx != dark->rx ||
+			brut->ry != dark->ry) {
+		return -1;
+	}
+
 	if (new_fit_image(&dark_tmp, dark->rx, dark->ry, 1))
 		return -1;
 	copyfits(dark, dark_tmp, CP_ALLOC | CP_EXTRACT, 0);
 
 	/* Minimization of background noise to find better k */
+	invalidate_stats_from_fit(brut);
 	k = goldenSectionSearch(brut, dark_tmp, lo, up, 1E-3);
 	if (k < 0.0)
 		return -1;
@@ -834,13 +855,17 @@ static int darkOptimization(fits *brut, fits *dark, fits *offset) {
 	siril_log_message(_("Dark optimization: %.3lf\n"), k);
 	/* Multiply coefficient to master-dark */
 	if (com.preprostatus & USE_OFFSET)
-		imoper(dark_tmp, offset, OPER_SUB);
+		ret = imoper(dark_tmp, offset, OPER_SUB);
+		if (ret) {
+			clearfits(dark_tmp);
+			return ret;
+		}
 	soper(dark_tmp, k, OPER_MUL);
-	imoper(brut, dark_tmp, OPER_SUB);
+	ret =  imoper(brut, dark_tmp, OPER_SUB);
 
 	clearfits(dark_tmp);
 
-	return 0;
+	return ret;
 }
 
 // idle function executed at the end of the sequence preprocessing
@@ -854,16 +879,20 @@ static gboolean end_sequence_prepro(gpointer p) {
 	gettimeofday(&t_end, NULL);
 	show_time(args->t_start, t_end);
 	update_used_memory();
-	if (!args->retval && !single_image_is_loaded()) {
-		// load the new sequence
-		char *ppseqname = malloc(
-				strlen(com.seq.ppprefix) + strlen(com.seq.seqname) + 5);
-		sprintf(ppseqname, "%s%s.seq", com.seq.ppprefix, com.seq.seqname);
-		check_seq(0);
-		update_sequences_list(ppseqname);
-		free(ppseqname);
+	if (args->is_sequence) {
+		if (!args->retval) {
+			// load the new sequence
+			char *ppseqname = malloc(
+					strlen(args->seq->ppprefix) + strlen(args->seq->seqname) + 5);
+			sprintf(ppseqname, "%s%s.seq", args->seq->ppprefix,
+					args->seq->seqname);
+			check_seq(0);
+			update_sequences_list(ppseqname);
+			free(ppseqname);
+		}
+		sequence_free_preprocessing_data(args->seq);
+		free(args->seq->ppprefix);
 	}
-	sequence_free_preprocessing_data(&com.seq);
 #ifdef MAC_INTEGRATION
 	GtkosxApplication *osx_app = gtkosx_application_get();
 	gtkosx_application_attention_request(osx_app, INFO_REQUEST);
@@ -877,60 +906,87 @@ static gboolean end_sequence_prepro(gpointer p) {
  * returns 1 on error */
 gpointer seqpreprocess(gpointer p) {
 	char dest_filename[256], msg[256];
-	fits *dark, *offset, *flat;
+	fits *dark, *offset, *flat, *fit = NULL;
+	int ret = 0;
 	struct preprocessing_data *args = (struct preprocessing_data *) p;
 
-	if (single_image_is_loaded()) {
-		dark = com.uniq->dark;
-		offset = com.uniq->offset;
-		flat = com.uniq->flat;
-	} else if (sequence_is_loaded()) {
-		dark = com.seq.dark;
-		offset = com.seq.offset;
-		flat = com.seq.flat;
-	} else
-		return GINT_TO_POINTER(1);
+	dark = args->dark;
+	offset = args->offset;
+	flat = args->flat;
+	args->retval = 0;
+
+	// remove old sequence
+	if (args->is_sequence) {
+		char *ppseqname = malloc(
+				strlen(args->seq->ppprefix) + strlen(args->seq->seqname) + 5);
+		sprintf(ppseqname, "%s%s.seq", args->seq->ppprefix, args->seq->seqname);
+		unlink(ppseqname);
+		free(ppseqname);
+	}
 
 	if (com.preprostatus & USE_FLAT) {
 		if (args->autolevel) {
 			/* TODO: evaluate the layer to apply but generally RLAYER is a good choice.
 			 * Indeed, if it is image from APN, CFA picture are in black & white */
-			imstats *stat = statistics(flat, RLAYER, NULL, STATS_BASIC, STATS_ZERO_NULLCHECK);
+			imstats *stat = statistics(NULL, -1, flat, RLAYER, NULL, STATS_BASIC);
 			if (!stat) {
-				siril_log_message(_("Error: no data computed.\n"));
+				siril_log_message(_("Error: statistics computation failed.\n"));
 				return GINT_TO_POINTER(1);
 			}
 			args->normalisation = stat->mean;
 			siril_log_message(_("Normalisation value auto evaluated: %.2lf\n"),
 					args->normalisation);
-			free(stat);
+			free_stats(stat);
 		}
 	}
 
-	if (single_image_is_loaded()) {
+	if (!args->is_sequence) {
 		snprintf(msg, 255, _("Pre-processing image %s"), com.uniq->filename);
 		msg[255] = '\0';
 		set_progress_bar_data(msg, 0.5);
 
-		if ((com.preprostatus & USE_OPTD) && (com.preprostatus & USE_DARK))
-			darkOptimization(com.uniq->fit, dark, offset);
+		if (new_fit_image(&fit, com.uniq->fit->rx, com.uniq->fit->ry,
+				com.uniq->fit->naxes[2]))
+			return GINT_TO_POINTER(1);
+		copyfits(com.uniq->fit, fit, CP_ALLOC | CP_FORMAT | CP_COPYA, 0);
+		copy_fits_metadata(com.uniq->fit, fit);
 
-		preprocess(com.uniq->fit, offset, dark, flat, args->normalisation);
+		if ((com.preprostatus & USE_OPTD) && (com.preprostatus & USE_DARK)) {
+			ret = darkOptimization(fit, dark, offset);
+			if (ret) {
+				set_progress_bar_data(msg, PROGRESS_NONE);
+				clearfits(fit);
+				free(fit);
+				return(GINT_TO_POINTER(1));
+			}
+		}
+
+		ret = preprocess(fit, offset, dark, flat, args->normalisation);
+		if (ret) {
+			set_progress_bar_data(msg, PROGRESS_NONE);
+			clearfits(fit);
+			free(fit);
+			return(GINT_TO_POINTER(1));
+		}
 
 		if ((com.preprostatus & USE_COSME) && (com.preprostatus & USE_DARK)) {
 			if (dark->naxes[2] == 1) {
-			/* Cosmetic correction */
-			long icold, ihot;
-			deviant_pixel *dev = find_deviant_pixels(dark, args->sigma, &icold, &ihot);
-			siril_log_message(_("%ld pixels corrected (%ld + %ld)\n"),
-					icold + ihot, icold, ihot);
-			cosmeticCorrection(com.uniq->fit, dev, icold + ihot, args->is_cfa);
-			if (dev)
-				free(dev);
+				/* Cosmetic correction */
+				long icold, ihot;
+				deviant_pixel *dev = find_deviant_pixels(dark, args->sigma, &icold, &ihot);
+				siril_log_message(_("%ld pixels corrected (%ld + %ld)\n"),
+						icold + ihot, icold, ihot);
+				cosmeticCorrection(fit, dev, icold + ihot, args->is_cfa);
+				if (dev)
+					free(dev);
 			}
 			else
 				siril_log_message(_("Darkmap cosmetic correction "
-						"is only supported with single channel images\n"));
+							"is only supported with single channel images\n"));
+		}
+
+		if (args->debayer) {
+			debayer_if_needed(TYPEFITS, fit, args->compatibility, TRUE);
 		}
 
 		gchar *filename = g_path_get_basename(com.uniq->filename);
@@ -940,7 +996,9 @@ gpointer seqpreprocess(gpointer p) {
 		snprintf(msg, 255, _("Saving image %s"), filename_noext);
 		msg[255] = '\0';
 		set_progress_bar_data(msg, PROGRESS_NONE);
-		savefits(dest_filename, com.uniq->fit);
+		savefits(dest_filename, fit);
+		clearfits(fit);
+		free(fit);
 		g_free(filename);
 		free(filename_noext);
 	} else {	// sequence
@@ -951,11 +1009,15 @@ gpointer seqpreprocess(gpointer p) {
 		deviant_pixel *dev = NULL;
 
 		// creating a SER file if the input data is SER
-		if (com.seq.type == SEQ_SER) {
+		if (args->seq->type == SEQ_SER) {
 			char new_ser_filename[256];
 			new_ser_file = calloc(1, sizeof(struct ser_struct));
-			snprintf(new_ser_filename, 255, "%s%s", com.seq.ppprefix, com.seq.ser_file->filename);
-			ser_create_file(new_ser_filename, new_ser_file, TRUE, com.seq.ser_file);
+			snprintf(new_ser_filename, 255, "%s%s", args->seq->ppprefix, args->seq->ser_file->filename);
+			if (ser_create_file(new_ser_filename, new_ser_file, TRUE, args->seq->ser_file)) {
+				free(new_ser_file);
+				new_ser_file = NULL;
+				return GINT_TO_POINTER(1);
+			}
 		}
 
 		if ((com.preprostatus & USE_COSME) && (com.preprostatus & USE_DARK)) {
@@ -968,45 +1030,64 @@ gpointer seqpreprocess(gpointer p) {
 						"is only supported with single channel images\n"));
 		}
 
-		fits *fit = calloc(1, sizeof(fits));
+		/* allocating memory to new fits */
+		fit = calloc(1, sizeof(fits));
+		if (!fit) {
+			fprintf(stderr, "Error: allocating memory to fit.\n");
+			siril_add_idle(end_sequence_prepro, args);
+			return GINT_TO_POINTER(1);
+		}
 
-		for (i = 0; i < com.seq.number; i++) {
+		for (i = 0; i < args->seq->number; i++) {
 			if (!get_thread_run())
 				break;
-			seq_get_image_filename(&com.seq, i, source_filename);
+			seq_get_image_filename(args->seq, i, source_filename);
 			snprintf(msg, 255, _("Loading and pre-processing image %d/%d (%s)"),
-					i + 1, com.seq.number, source_filename);
+					i + 1, args->seq->number, source_filename);
 			msg[255] = '\0';
 			set_progress_bar_data(msg,
-					(double) (i + 1) / (double) com.seq.number);
-			if (seq_read_frame(&com.seq, i, fit)) {
+					(double) (i + 1) / (double) args->seq->number);
+			if (seq_read_frame(args->seq, i, fit)) {
 				snprintf(msg, 255, _("Could not read one of the raw files: %s."
 						" Aborting preprocessing."), source_filename);
 				msg[255] = '\0';
 				set_progress_bar_data(msg, PROGRESS_RESET);
 				args->retval = 1;
-				if (com.seq.type == SEQ_SER) {
-					ser_write_and_close(new_ser_file);
-					free(new_ser_file);
-					new_ser_file = NULL;
+				break;
+			}
+			if ((com.preprostatus & USE_OPTD) && (com.preprostatus & USE_DARK)) {
+				ret = darkOptimization(fit, dark, offset);
+				if (ret) {
+					set_progress_bar_data(msg, PROGRESS_NONE);
+					clearfits(fit);
+					free(fit);
+					siril_add_idle(end_sequence_prepro, args);
+					return GINT_TO_POINTER(ret);
 				}
-				gdk_threads_add_idle(end_sequence_prepro, args);
-				return GINT_TO_POINTER(1);
 			}
-			if ((com.preprostatus & USE_OPTD) && (com.preprostatus & USE_DARK))
-				darkOptimization(fit, dark, offset);
 
-			preprocess(fit, offset, dark, flat, args->normalisation);
+			ret = preprocess(fit, offset, dark, flat, args->normalisation);
+			if (ret) {
+				set_progress_bar_data(msg, PROGRESS_NONE);
+				clearfits(fit);
+				free(fit);
+				siril_add_idle(end_sequence_prepro, args);
+				return GINT_TO_POINTER(ret);
+			}
 
 			if ((com.preprostatus & USE_COSME) && (com.preprostatus & USE_DARK) && (dark->naxes[2] == 1))
 				cosmeticCorrection(fit, dev, icold + ihot, args->is_cfa);
 
-			snprintf(dest_filename, 255, "%s%s", com.seq.ppprefix,
+			if (args->debayer && args->seq->type == SEQ_REGULAR) {
+				debayer_if_needed(TYPEFITS, fit, args->compatibility, TRUE);
+			}
+
+			snprintf(dest_filename, 255, "%s%s", args->seq->ppprefix,
 					source_filename);
 			dest_filename[255] = '\0';
-			snprintf(msg, 255, "Saving image %d/%d (%s)", i + 1, com.seq.number,
+			snprintf(msg, 255, "Saving image %d/%d (%s)", i + 1, args->seq->number,
 					dest_filename);
-			if (com.seq.type == SEQ_SER) {
+			if (args->seq->type == SEQ_SER) {
 				ser_write_frame_from_fit(new_ser_file, fit, i);
 			} else {
 				savefits(dest_filename, fit);
@@ -1015,7 +1096,7 @@ gpointer seqpreprocess(gpointer p) {
 		}
 		free(fit);
 		// closing SER file if it applies
-		if (com.seq.type == SEQ_SER && (new_ser_file != NULL)) {
+		if (args->seq->type == SEQ_SER && (new_ser_file != NULL)) {
 			ser_write_and_close(new_ser_file);
 			free(new_ser_file);
 			new_ser_file = NULL;
@@ -1023,9 +1104,8 @@ gpointer seqpreprocess(gpointer p) {
 		set_progress_bar_data(PROGRESS_TEXT_RESET, PROGRESS_RESET);
 		if (dev) free(dev);
 	}
-	args->retval = 0;
-	gdk_threads_add_idle(end_sequence_prepro, args);
-	return GINT_TO_POINTER(0);
+	siril_add_idle(end_sequence_prepro, args);
+	return GINT_TO_POINTER(args->retval);
 }
 
 /* computes the background value using the histogram and/or median value.
@@ -1039,15 +1119,13 @@ double background(fits* fit, int reqlayer, rectangle *selection) {
 	else if (isrgb(&gfit))
 		layer = GLAYER;		//GLAYER is better to evaluate background
 
-	imstats* stat = statistics(fit, layer, selection, STATS_BASIC, STATS_ZERO_NULLCHECK);
+	imstats* stat = statistics(NULL, -1, fit, layer, selection, STATS_BASIC);
 	if (!stat) {
-		siril_log_message(_("Error: no data computed.\n"));
+		siril_log_message(_("Error: statistics computation failed.\n"));
 		return 0.0;
 	}
 	bg = stat->median;
-
-	free(stat);
-	stat = NULL;
+	free_stats(stat);
 	return bg;
 }
 
@@ -1140,14 +1218,10 @@ int verbose_rotate_image(fits *image, double angle, int interpolation,
  * extracts plan "Plan" in fit parameters */
 
 int get_wavelet_layers(fits *fit, int Nbr_Plan, int Plan, int Type, int reqlayer) {
-	char *File_Name_Transform[3] = { "r_rawdata.wave", "g_rawdata.wave",
-			"b_rawdata.wave" }, *dir[3];
-	const char *tmpdir;
-	int chan, start, end;
-	wave_transf_des Wavelet[3];
+	int chan, start, end, retval = 0;
+	wave_transf_des wavelet[3];
 
 	assert(fit->naxes[2] <= 3);
-	tmpdir = g_get_tmp_dir();
 
 	float *Imag = f_vector_alloc(fit->ry * fit->rx);
 	if (Imag == NULL)
@@ -1161,34 +1235,42 @@ int get_wavelet_layers(fits *fit, int Nbr_Plan, int Plan, int Type, int reqlayer
 		start = reqlayer;
 		end = start + 1;
 	}
+
 	for (chan = start; chan < end; chan++) {
 		int Nl, Nc;
 
-		dir[chan] = g_build_filename(tmpdir, File_Name_Transform[chan], NULL);
-		if (wavelet_transform_file(Imag, fit->ry, fit->rx, dir[chan], Type, Nbr_Plan,
-				fit->pdata[chan])) {
-			free((char *) Imag);
-			g_free(dir[chan]);
-			return 1;
+		if (wavelet_transform(Imag, fit->ry, fit->rx, &wavelet[chan],
+					Type, Nbr_Plan, fit->pdata[chan])) {
+			retval = 1;
+			break;
 		}
-		if (wave_io_read(dir[chan], &Wavelet[chan])) {
-			free((char *) Imag);
-			g_free(dir[chan]);
-			return 1;
-		}
-		Nl = Wavelet[chan].Nbr_Ligne;
-		Nc = Wavelet[chan].Nbr_Col;
-		pave_2d_extract_plan(Wavelet[chan].Pave.Data, Imag, Nl, Nc, Plan);
+		Nl = wavelet[chan].Nbr_Ligne;
+		Nc = wavelet[chan].Nbr_Col;
+		pave_2d_extract_plan(wavelet[chan].Pave.Data, Imag, Nl, Nc, Plan);
 		reget_rawdata(Imag, Nl, Nc, fit->pdata[chan]);
-		g_free(dir[chan]);
+		wave_io_free(&wavelet[chan]);
 	}
 
 	/* Free */
-	if (Imag)
-		free((char *) Imag);
-	for (chan = start; chan < end; chan++) {
-		wave_io_free(&Wavelet[chan]);
+	free(Imag);
+	return retval;
+}
+
+int extract_plans(fits *fit, int Nbr_Plan, int Type) {
+	int i;
+
+	set_progress_bar_data(PROGRESS_TEXT_RESET, PROGRESS_RESET);
+
+	for (i = 0; i < Nbr_Plan; i++) {
+		char filename[256], msg[256];
+
+		g_snprintf(filename, sizeof(filename), "layer%02d", i);
+		snprintf(msg, 256, _("Extracting %s..."), filename);
+		set_progress_bar_data(msg, (float)i / Nbr_Plan);
+		get_wavelet_layers(fit, Nbr_Plan, i, Type, -1);
+		savefits(filename, fit);
 	}
+	set_progress_bar_data(PROGRESS_TEXT_RESET, PROGRESS_DONE);
 	return 0;
 }
 
@@ -1219,23 +1301,23 @@ gpointer median_filter(gpointer p) {
 	int ny = args->fit->ry;
 	int radius = (args->ksize - 1) / 2;
 	double norm = (double) get_normalized_value(args->fit);
-
+	double cur, total;
 	assert(nx > 0 && ny > 0);
 
 	struct timeval t_start, t_end;
 
-	siril_log_color_message(_("Median Filter: processing...\n"), "red");
+	char *msg = siril_log_color_message(_("Median Filter: processing...\n"), "red");
+	msg[strlen(msg) - 1] = '\0';
+	set_progress_bar_data(msg, PROGRESS_RESET);
 	gettimeofday(&t_start, NULL);
 
 	do {
-		if (args->iterations != 1)
-			siril_log_message(_("Iteration #%d...\n"), iter + 1);
-		for (layer = 0; layer < com.uniq->nb_layers; layer++) {
+		for (layer = 0; layer < args->fit->naxes[2]; layer++) {
 			/* FILL image upside-down */
 			WORD **image = malloc(ny * sizeof(WORD *));
 			if (image == NULL) {
 				printf("median filter: error allocating data\n");
-				gdk_threads_add_idle(end_median_filter, args);
+				siril_add_idle(end_median_filter, args);
 				return GINT_TO_POINTER(1);
 			}
 			for (i = 0; i < ny; i++)
@@ -1244,13 +1326,18 @@ gpointer median_filter(gpointer p) {
 			for (y = 0; y < ny; y++) {
 				if (!get_thread_run())
 					break;
+				total = ny * args->iterations;
+				cur = (double) y + (ny * iter);
+				if (y & 4)	// every 16 iterations
+					set_progress_bar_data(NULL, cur / total);
 				for (x = 0; x < nx; x++) {
 					WORD *data = calloc(args->ksize * args->ksize,
 							sizeof(WORD));
 					if (data == NULL) {
 						printf("median filter: error allocating data\n");
 						free(image);
-						gdk_threads_add_idle(end_median_filter, args);
+						siril_add_idle(end_median_filter, args);
+						set_progress_bar_data(_("Median filter failed"), PROGRESS_DONE);
 						return GINT_TO_POINTER(1);
 					}
 					i = 0;
@@ -1295,9 +1382,11 @@ gpointer median_filter(gpointer p) {
 		}
 		iter++;
 	} while (iter < args->iterations && get_thread_run());
+	invalidate_stats_from_fit(args->fit);
 	gettimeofday(&t_end, NULL);
 	show_time(t_start, t_end);
-	gdk_threads_add_idle(end_median_filter, args);
+	set_progress_bar_data(_("Median filter applied"), PROGRESS_DONE);
+	siril_add_idle(end_median_filter, args);
 
 	return GINT_TO_POINTER(0);
 }
@@ -1312,6 +1401,7 @@ static int fmul_layer(fits *a, int layer, float coeff) {
 	for (i = 0; i < a->rx * a->ry; ++i) {
 		buf[i] = round_to_WORD(buf[i] * coeff);
 	}
+	invalidate_stats_from_fit(a);
 	return 0;
 }
 
@@ -1319,7 +1409,7 @@ static int fmul_layer(fits *a, int layer, float coeff) {
  *      B A N D I N G      R E D U C T I O N      M A N A G E M E N T        *
  ****************************************************************************/
 
-int banding_image_hook(struct generic_seq_args *args, int i, fits *fit, rectangle *_) {
+int banding_image_hook(struct generic_seq_args *args, int o, int i, fits *fit, rectangle *_) {
 	struct banding_data *banding_args = (struct banding_data *)args->user;
 	return BandingEngine(fit, banding_args->sigma, banding_args->amount,
 			banding_args->protect_highlights, banding_args->applyRotation);
@@ -1379,13 +1469,13 @@ gpointer BandingEngineThreaded(gpointer p) {
 
 	gettimeofday(&t_end, NULL);
 	show_time(t_start, t_end);
-	gdk_threads_add_idle(end_BandingEngine, args);
+	siril_add_idle(end_BandingEngine, args);
 	
 	return GINT_TO_POINTER(retval);
 }
 
 int BandingEngine(fits *fit, double sigma, double amount, gboolean protect_highlights, gboolean applyRotation) {
-	int chan, row, i;
+	int chan, row, i, ret = 0;
 	WORD *line, *fixline;
 	double minimum = DBL_MAX, globalsigma = 0.0;
 	fits *fiximage = NULL;
@@ -1399,27 +1489,27 @@ int BandingEngine(fits *fit, double sigma, double amount, gboolean protect_highl
 		return 1;
 
 	for (chan = 0; chan < fit->naxes[2]; chan++) {
-		imstats *stat = statistics(fit, chan, NULL, STATS_BASIC | STATS_MAD, STATS_ZERO_NULLCHECK);
+		imstats *stat = statistics(NULL, -1, fit, chan, NULL, STATS_BASIC | STATS_MAD);
 		if (!stat) {
-			siril_log_message(_("Error: no data computed.\n"));
+			siril_log_message(_("Error: statistics computation failed.\n"));
 			return 1;
 		}
 		double background = stat->median;
 		double *rowvalue = calloc(fit->ry, sizeof(double));
 		if (rowvalue == NULL) {
 			fprintf(stderr, "BandingEngine: error allocating data\n");
-			free(stat);
+			free_stats(stat);
 			return 1;
 		}
 		if (protect_highlights) {
 			globalsigma = stat->mad * MAD_NORM;
 		}
+		free_stats(stat);
 		for (row = 0; row < fit->ry; row++) {
 			line = fit->pdata[chan] + row * fit->rx;
 			WORD *cpyline = calloc(fit->rx, sizeof(WORD));
 			if (cpyline == NULL) {
 				fprintf(stderr, "BandingEngine: error allocating data\n");
-				free(stat);
 				free(rowvalue);
 				return 1;
 			}
@@ -1447,16 +1537,16 @@ int BandingEngine(fits *fit, double sigma, double amount, gboolean protect_highl
 				fixline[i] = round_to_WORD(rowvalue[row] - minimum);
 		}
 		free(rowvalue);
-		free(stat);
 	}
 	for (chan = 0; chan < fit->naxes[2]; chan++)
 		fmul_layer(fiximage, chan, amount);
-	imoper(fit, fiximage, OPER_ADD);
+	ret = imoper(fit, fiximage, OPER_ADD);
 
+	invalidate_stats_from_fit(fit);
 	clearfits(fiximage);
-	if (applyRotation)
+	if ((!ret) && applyRotation)
 		cvRotateImage(fit, -90.0, -1, OPENCV_LINEAR);
-	return 0;
+	return ret;
 }
 
 /*****************************************************************************
@@ -1481,19 +1571,24 @@ int backgroundnoise(fits* fit, double sigma[]) {
 	cvComputeFinestScale(waveimage);
 
 	for (layer = 0; layer < fit->naxes[2]; layer++) {
-		imstats *stat = statistics(waveimage, layer, NULL, STATS_BASIC, STATS_ZERO_NULLCHECK);
-		if (!stat) {
-			siril_log_message(_("Error: no data computed.\n"));
-			return 1;
-		}
-		double sigma0 = stat->sigma;
-		double mean = stat->mean;
+		double sigma0, mean, norm_val;
 		double epsilon = 0.0;
 		WORD lo, hi;
 		WORD *buf = waveimage->pdata[layer];
 		unsigned int i;
 		unsigned int ndata = fit->rx * fit->ry;
 		assert(ndata > 0);
+
+		imstats *stat = statistics(NULL, -1, waveimage, layer, NULL, STATS_BASIC);
+		if (!stat) {
+			siril_log_message(_("Error: statistics computation failed.\n"));
+			return 1;
+		}
+		sigma0 = stat->sigma;
+		mean = stat->mean;
+		norm_val = stat->normValue;
+		free_stats(stat);
+
 		WORD *array1 = calloc(ndata, sizeof(WORD));
 		WORD *array2 = calloc(ndata, sizeof(WORD));
 		if (array1 == NULL || array2 == NULL) {
@@ -1502,14 +1597,13 @@ int backgroundnoise(fits* fit, double sigma[]) {
 				free(array1);
 			if (array2)
 				free(array2);
-			free(stat);
 			return 1;
 		}
 		WORD *set = array1, *subset = array2;
 		memcpy(set, buf, ndata * sizeof(WORD));
 
-		lo = round_to_WORD(LOW_BOUND * stat->normValue);
-		hi = round_to_WORD(HIGH_BOUND * stat->normValue);
+		lo = round_to_WORD(LOW_BOUND * norm_val);
+		hi = round_to_WORD(HIGH_BOUND * norm_val);
 
 		sigma[layer] = sigma0;
 
@@ -1530,7 +1624,6 @@ int backgroundnoise(fits* fit, double sigma[]) {
 			if (ndata == 0) {
 				free(array1);
 				free(array2);
-				free(stat);
 				siril_log_message(_("backgroundnoise: Error, no data computed\n"));
 				sigma[layer] = 0.0;
 				return 1;
@@ -1544,26 +1637,17 @@ int backgroundnoise(fits* fit, double sigma[]) {
 			siril_log_message(_("backgroundnoise: does not converge\n"));
 		free(array1);
 		free(array2);
-		free(stat);
 	}
 	clearfits(waveimage);
+	invalidate_stats_from_fit(fit);
 
 	return 0;
 }
 
 gboolean end_noise(gpointer p) {
 	struct noise_data *args = (struct noise_data *) p;
-	stop_processing_thread();// can it be done here in case there is no thread?
-	int chan, nb_chan;
+	stop_processing_thread();
 	struct timeval t_end;
-
-	nb_chan = args->fit->naxes[2];
-
-	double norm = (double) get_normalized_value(args->fit);
-	for (chan = 0; chan < nb_chan; chan++)
-		siril_log_message(
-				_("Background noise value (channel: #%d): %0.3lf (%.3e)\n"), chan,
-				args->bgnoise[chan], args->bgnoise[chan] / norm);
 	set_cursor_waiting(FALSE);
 	update_used_memory();
 	if (args->verbose) {
@@ -1576,7 +1660,8 @@ gboolean end_noise(gpointer p) {
 
 gpointer noise(gpointer p) {
 	struct noise_data *args = (struct noise_data *) p;
-	int  chan;
+	int chan;
+	args->retval = 0;
 
 	if (args->verbose) {
 		siril_log_color_message(_("Noise standard deviation: calculating...\n"),
@@ -1584,25 +1669,28 @@ gpointer noise(gpointer p) {
 		gettimeofday(&args->t_start, NULL);
 	}
 
-/*	if (backgroundnoise(args->fit, args->bgnoise)) {
-		gdk_threads_add_idle(end_noise, args);
-		return GINT_TO_POINTER(1);
-	}
-	*/
 	for (chan = 0; chan < args->fit->naxes[2]; chan++) {
-		imstats *stat = statistics(args->fit, chan, NULL, STATS_BASIC, STATS_ZERO_NULLCHECK);
+		imstats *stat = statistics(NULL, -1, args->fit, chan, NULL, STATS_NOISE);
 		if (!stat) {
-			siril_log_message(_("Error: no data computed.\n"));
-			gdk_threads_add_idle(end_noise, args);
-			return GINT_TO_POINTER(1);
+			args->retval = 1;
+			siril_log_message(_("Error: statistics computation failed.\n"));
+			break;
 		}
 		args->bgnoise[chan] = stat->bgnoise;
-		free(stat);
+		free_stats(stat);
 	}
 
-	gdk_threads_add_idle(end_noise, args);
+	if (!args->retval) {
+		double norm = (double) get_normalized_value(args->fit);
+		for (chan = 0; chan < args->fit->naxes[2]; chan++)
+			siril_log_message(
+					_("Background noise value (channel: #%d): %0.3lf (%.3e)\n"), chan,
+					args->bgnoise[chan], args->bgnoise[chan] / norm);
+	}
 
-	return GINT_TO_POINTER(0);
+	siril_add_idle(end_noise, args);
+
+	return GINT_TO_POINTER(args->retval);
 }
 
 gpointer LRdeconv(gpointer p) {
@@ -1617,7 +1705,7 @@ gpointer LRdeconv(gpointer p) {
 	gettimeofday(&t_end, NULL);
 	show_time(t_start, t_end);
 
-	gdk_threads_add_idle(end_generic, args);
+	siril_add_idle(end_generic, args);
 	adjust_cutoff_from_updated_gfit();
 	redraw(com.cvport, REMAP_ALL);
 	redraw_previews();
