@@ -189,7 +189,7 @@ static gchar *parse_changelog(gchar *changelog) {
 		strResult = g_string_append(strResult, "\n");
 		i++;
 	}
-
+	g_strfreev(token);
 	return g_string_free(strResult, FALSE);
 }
 
@@ -248,18 +248,18 @@ static gboolean end_update_idle(gpointer p) {
 	gchar *changelog = NULL;
 	const char *data = NULL;
 	version_number current_version, last_version_available;
-	static char *icon[] = { "dialog-information", "dialog-error" };
+	static char *icon[] = { "dialog-information-symbolic", "dialog-error-symbolic" };
 	struct _update_data *args = (struct _update_data *) p;
 
 	if (args->content == NULL) {
 		switch(args->code) {
 		case 0:
-			msg = siril_log_message(_("Unable to check updates !! "
+			msg = siril_log_message(_("Unable to check updates! "
 					"Please Check your network connection\n"));
 			break;
 		default:
-		msg = siril_log_message(_("Unable to check updates !! Error: %ld\n"),
-				args->code);
+			msg = siril_log_message(_("Unable to check updates! Error: %ld\n"),
+					args->code);
 		}
 		ret = 1;
 	} else {
