@@ -94,7 +94,8 @@ gboolean redraw_preview(GtkWidget *widget, cairo_t *cr, gpointer data) {
 
 void redraw_previews() {
 	int i;
-	for (i=0; i<PREVIEW_NB; i++)
+	if (com.script) return;
+	for (i = 0; i < PREVIEW_NB; i++)
 		gtk_widget_queue_draw(com.preview_area[i]);
 }
 
@@ -233,7 +234,7 @@ void on_spinbut_shift_value_change(GtkSpinButton *spinbutton, gpointer user_data
 		com.seq.regparam[current_layer][com.seq.current].shiftx = (float) new_value;
 	else com.seq.regparam[current_layer][com.seq.current].shifty = (float) new_value;
 	writeseqfile(&com.seq);
-	fill_sequence_list(&com.seq, current_layer);	// update list with new regparam
+	fill_sequence_list(&com.seq, current_layer, FALSE);	// update list with new regparam
 	redraw_previews();
 }
 
