@@ -89,7 +89,6 @@ enum {
 	N_COLUMNS
 };
 
-static gpointer convert_thread_worker(gpointer p);
 static gboolean end_convert_idle(gpointer p);
 
 
@@ -151,30 +150,8 @@ static gint sort_conv_tree(GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b,
 	return ret;
 }
 
-void check_for_conversion_form_completeness() {
-	static GtkTreeView *tree_convert = NULL;
-	GtkTreeIter iter;
-	GtkTreeModel *model = NULL;
-	gboolean valid;
-	GtkWidget *go_button = lookup_widget("convert_button");
-	
-	if (tree_convert == NULL)
-		tree_convert = GTK_TREE_VIEW(gtk_builder_get_object(builder, "treeview_convert"));
-	
-	model = gtk_tree_view_get_model(tree_convert);
-	valid = gtk_tree_model_get_iter_first(model, &iter);
-	gtk_widget_set_sensitive (go_button, destroot && destroot[0] != '\0' && valid);
-
-	/* we override the sort function in order to provide natural sort order */
-	gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(model),
-			COLUMN_CONV_FILENAME, (GtkTreeIterCompareFunc) sort_conv_tree, NULL,
-			NULL);
-
-	update_statusbar_convert();
-}
-
 // input is destroot
-char *create_sequence_filename(int counter, char *output, int outsize) {
+static char *create_sequence_filename(int counter, char *output, int outsize) {
 	const char *ext = get_filename_ext(destroot);
 	if (ext) {
 		/* we need to insert a number before the extension */
@@ -200,67 +177,6 @@ char *create_sequence_filename(int counter, char *output, int outsize) {
 	return output;
 }
 
-// truncates destroot if it's more than 120 characters, append a '_' if it
-// doesn't end with one or a '-'. SER extensions are accepted and unmodified.
-void on_convtoroot_changed (GtkEditable *editable, gpointer user_data){
-	static GtkWidget *multiple_ser = NULL;
-	const gchar *name = gtk_entry_get_text(GTK_ENTRY(editable));
-	if (!multiple_ser)
-		multiple_ser = lookup_widget("multipleSER");
-	if (destroot) g_free(destroot);
-
-	destroot = g_str_to_ascii(name, NULL); // we want to avoid special char
-
-	const char *ext = get_filename_ext(destroot);
-	if (ext && !g_ascii_strcasecmp(ext, "ser")) {
-		convflags |= CONVDSTSER;
-		gtk_widget_set_visible(multiple_ser, TRUE);
-		return;
-	}
-	gtk_widget_set_visible(multiple_ser, FALSE);
-
-	destroot = format_basename(destroot);
-
-	check_for_conversion_form_completeness();
-}
-
-void on_demosaicing_toggled (GtkToggleButton *togglebutton, gpointer user_data) {
-	static GtkToggleButton *but = NULL;
-	if (!but) but = GTK_TOGGLE_BUTTON(lookup_widget("radiobutton1"));
-	if (gtk_toggle_button_get_active(togglebutton)) {
-		convflags |= CONVDEBAYER;
-		gtk_toggle_button_set_active(but, TRUE);
-		com.debayer.open_debayer = TRUE;
-	}
-	else {
-		convflags &= ~CONVDEBAYER;	// used for conversion
-		com.debayer.open_debayer = FALSE;	// used for image opening
-	}
-}
-
-void on_multipleSER_toggled (GtkToggleButton *togglebutton, gpointer user_data) {
-	if (gtk_toggle_button_get_active(togglebutton))
-		convflags |= CONVMULTIPLE;
-	else convflags &= ~CONVMULTIPLE;
-}
-
-void on_conv3planefit_toggled (GtkToggleButton *togglebutton, gpointer user_data){
-	convflags |= CONV1X3;
-	convflags &= ~(CONV3X1|CONV1X1);
-}
-
-void on_conv3_1plane_toggled (GtkToggleButton *togglebutton, gpointer user_data) {
-	convflags |= CONV3X1;
-	convflags &= ~(CONV1X1|CONV1X3);
-}
-
-void on_conv1_1plane_toggled (GtkToggleButton *togglebutton, gpointer user_data) {
-	convflags |= CONV1X1;
-	convflags &= ~(CONV3X1|CONV1X3);
-}
-
-/*************************************************************************************/
-
 /* This function sets all default values of libraw settings in the com.raw_set
  * struct, as defined in the glade file.
  * When the ini file is read, the values of com.raw_set are overwritten, but if the
@@ -268,28 +184,144 @@ void on_conv1_1plane_toggled (GtkToggleButton *togglebutton, gpointer user_data)
  * GUI states reset to zero by set_GUI_LIBRAW() because the data in com.raw_set had
  * not been initialized with the default GUI values (= initialized to 0).
  */
-void initialize_libraw_settings() {
+static void initialize_libraw_settings() {
 	com.raw_set.bright = 1.0;		// brightness
 	com.raw_set.mul[0] = 1.0;		// multipliers: red
 	com.raw_set.mul[1] = 1.0;		// multipliers: green, not used because always equal to 1
 	com.raw_set.mul[2] = 1.0;		// multipliers: blue
 	com.raw_set.auto_mul = 1;		// multipliers are Either read from file, or calculated on the basis of file data, or taken from hardcoded constants
 	com.raw_set.user_black = 0;		// black point correction
-	com.raw_set.use_camera_wb = 0;	// if possible, use the white balance from the camera. 
+	com.raw_set.use_camera_wb = 0;	// if possible, use the white balance from the camera.
 	com.raw_set.use_auto_wb = 0;		// use automatic white balance obtained after averaging over the entire image
 	com.raw_set.user_qual = 1;		// type of interpolation. AHD by default
 	com.raw_set.gamm[0] = 1.0;		// gamma curve: linear by default
 	com.raw_set.gamm[1] = 1.0;
 }
 
-void initialize_ser_debayer_settings() {
+static void initialize_ser_debayer_settings() {
 	com.debayer.open_debayer = FALSE;
 	com.debayer.use_bayer_header = TRUE;
 	com.debayer.compatibility = FALSE;
 	com.debayer.bayer_pattern = BAYER_FILTER_RGGB;
 	com.debayer.bayer_inter = BAYER_VNG;
 }
- 
+
+static gboolean end_convert_idle(gpointer p) {
+	struct _convert_data *args = (struct _convert_data *) p;
+	struct timeval t_end;
+
+	if (get_thread_run() && args->nb_converted > 1) {
+		// load the sequence
+		char *ppseqname = malloc(strlen(args->destroot) + 5);
+		sprintf(ppseqname, "%s.seq", args->destroot);
+		check_seq(0);
+		update_sequences_list(ppseqname);
+		free(ppseqname);
+	}
+	update_used_memory();
+	set_progress_bar_data(PROGRESS_TEXT_RESET, PROGRESS_DONE);
+	set_cursor_waiting(FALSE);
+	gettimeofday(&t_end, NULL);
+	show_time(args->t_start, t_end);
+	stop_processing_thread();
+	g_free(args->destroot);
+	free(args);
+	return FALSE;
+}
+
+/* from a fits object, save to file or files, based on the channel policy from convflags */
+static int save_to_target_fits(fits *fit, const char *dest_filename) {
+	if (convflags & CONV3X1) {	// an RGB image to 3 fits, one for each channel
+		char filename[130];
+
+		if (fit->naxis != 3) {
+			siril_log_message(_("Saving to 3 FITS files cannot be done because the source image does not have three channels\n"));
+			return 1;
+		}
+		sprintf(filename, "r_%s", dest_filename);
+		if (save1fits16(filename, fit, RLAYER)) {
+			printf("tofits: save1fit8 error, CONV3X1\n");
+			return 1;
+		}
+		sprintf(filename, "g_%s", dest_filename);
+		if (save1fits16(filename, fit, GLAYER)) {
+			printf("tofits: save1fit8 error, CONV3X1\n");
+			return 1;
+		}
+		sprintf(filename, "b_%s", dest_filename);
+		if (save1fits16(filename, fit, BLAYER)) {
+			printf("tofits: save1fit8 error, CONV3X1\n");
+			return 1;
+		}
+	} else if (convflags & CONV1X1) { // a single FITS to convert from an RGB grey image
+		if (save1fits16(dest_filename, fit, RLAYER)) {
+			printf("tofits: save1fit8 error, CONV1X1\n");
+			return 1;
+		}
+	} else {			// normal FITS save, any format
+		if (savefits(dest_filename, fit)) {
+			printf("tofits: savefit error, CONV1X3\n");
+			return 1;
+		}
+	}
+	return 0;
+}
+
+static int retrieveBayerPattern(char *bayer) {
+	int i;
+
+	for (i = 0; i < (sizeof(filter_pattern) / sizeof(char *)); i++) {
+		if (g_ascii_strcasecmp(bayer, filter_pattern[i]) == 0) {
+			return i;
+		}
+	}
+	return BAYER_FILTER_NONE;
+}
+
+/* open the file with path source from any image type and load it into a new FITS object */
+static fits *any_to_new_fits(image_type imagetype, const char *source, gboolean compatibility) {
+	int retval = 0;
+	fits *tmpfit = calloc(1, sizeof(fits));
+
+	retval = any_to_fits(imagetype, source, tmpfit);
+
+	if (!retval)
+		retval = debayer_if_needed(imagetype, tmpfit, compatibility, FALSE);
+
+	if (retval) {
+		clearfits(tmpfit);
+		free(tmpfit);
+		return NULL;
+	}
+
+	return tmpfit;
+}
+
+/**************************Public functions***********************************************************/
+
+void check_for_conversion_form_completeness() {
+	static GtkTreeView *tree_convert = NULL;
+	GtkTreeIter iter;
+	GtkTreeModel *model = NULL;
+	gboolean valid;
+	GtkWidget *go_button = lookup_widget("convert_button");
+
+	if (tree_convert == NULL)
+		tree_convert = GTK_TREE_VIEW(gtk_builder_get_object(builder, "treeview_convert"));
+
+	model = gtk_tree_view_get_model(tree_convert);
+	valid = gtk_tree_model_get_iter_first(model, &iter);
+	gtk_widget_set_sensitive (go_button, destroot && destroot[0] != '\0' && valid);
+
+	/* we override the sort function in order to provide natural sort order */
+	gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(model),
+			COLUMN_CONV_FILENAME, (GtkTreeIterCompareFunc) sort_conv_tree, NULL,
+			NULL);
+
+	update_statusbar_convert();
+}
+
+
 /* initialize converters (utilities used for different image types importing) *
  * updates the label listing the supported input file formats, and modifies the
  * list of file types used in convflags */
@@ -331,7 +363,7 @@ void initialize_converters() {
 	supported_filetypes |= TYPERAW;
 	g_string_append(string, ", ");
 	g_string_append(string, _("RAW images"));
-	set_libraw_settings_menu_available(TRUE);	// enable libraw settings
+	if (!com.script) set_libraw_settings_menu_available(TRUE);	// enable libraw settings
 	initialize_libraw_settings();	// below in the file
 	
 	nb_raw = get_nb_raw_supported();
@@ -342,7 +374,7 @@ void initialize_converters() {
 	}
 	count_ext += nb_raw;
 #else
-	set_libraw_settings_menu_available(FALSE);	// disable libraw settings
+	if (!com.script) set_libraw_settings_menu_available(FALSE);	// disable libraw settings
 #endif
 	g_string_append(string, ", ");
 	g_string_append(string, _("FITS-CFA images"));
@@ -383,9 +415,11 @@ void initialize_converters() {
 	supported_extensions[count_ext++] = NULL;		// NULL-terminated array
 
 	g_string_append(string, ".");
-	label_supported = GTK_LABEL(gtk_builder_get_object(builder, "label_supported_types"));
 	text = g_string_free(string, FALSE);
-	gtk_label_set_text(label_supported, text);
+	if (!com.script) {
+		label_supported = GTK_LABEL(gtk_builder_get_object(builder, "label_supported_types"));
+		gtk_label_set_text(label_supported, text);
+	}
 	siril_log_message(_("Supported file types: %s\n"), text + 1);
 	g_free(text);
 }
@@ -459,15 +493,6 @@ int count_selected_files() {
 	return count;
 }
 
-struct _convert_data {
-	struct timeval t_start;
-	GDir *dir;
-	GList *list;
-	int start;
-	int total;
-	int nb_converted;
-	gboolean compatibility;
-};
 
 void on_convert_button_clicked(GtkButton *button, gpointer user_data) {
 	GDir *dir;
@@ -521,13 +546,13 @@ void on_convert_button_clicked(GtkButton *button, gpointer user_data) {
 	char *tmpmsg;
 	if (!com.wd) {
 		tmpmsg = siril_log_message(_("Conversion: no working directory set.\n"));
-		show_dialog(tmpmsg, _("Warning"), "dialog-warning");
+		show_dialog(tmpmsg, _("Warning"), "dialog-warning-symbolic");
 		set_cursor_waiting(FALSE);
 		return;
 	}
 	if((dir = g_dir_open(com.wd, 0, &error)) == NULL){
 		tmpmsg = siril_log_message(_("Conversion: error opening working directory %s.\n"), com.wd);
-		show_dialog(tmpmsg, _("Error"), "dialog-error");
+		show_dialog(tmpmsg, _("Error"), "dialog-error-symbolic");
 		fprintf (stderr, "Conversion: %s\n", error->message);
 		set_cursor_waiting(FALSE);
 		return ;
@@ -542,11 +567,13 @@ void on_convert_button_clicked(GtkButton *button, gpointer user_data) {
 	args->t_start.tv_sec = t_start.tv_sec;
 	args->t_start.tv_usec = t_start.tv_usec;
 	args->compatibility = com.debayer.compatibility;
+	args->command_line = FALSE;
+	args->destroot = g_strdup(destroot);
 	start_in_new_thread(convert_thread_worker, args);
 	return;
 }
 
-static gpointer convert_thread_worker(gpointer p) {
+gpointer convert_thread_worker(gpointer p) {
 	char dest_filename[128], msg_bar[256];
 	int indice;
 	double progress = 0.0;
@@ -564,7 +591,7 @@ static gpointer convert_thread_worker(gpointer p) {
 		} else {
 			ser_file = malloc(sizeof(struct ser_struct));
 			if (!(convflags & CONVMULTIPLE)) {
-				if (ser_create_file(destroot, ser_file, TRUE, NULL)) {
+				if (ser_create_file(args->destroot, ser_file, TRUE, NULL)) {
 					siril_log_message(_("Creating the SER file failed, aborting.\n"));
 					goto clean_exit;
 				}
@@ -597,7 +624,7 @@ static gpointer convert_thread_worker(gpointer p) {
 				"If the file type you are trying to load is listed in supported "
 				"formats, you may notify the developpers that the extension you are "
 				"trying to use should be recognized for this type."), src_ext);
-			show_dialog(msg, _("Error"), "dialog-error");
+			show_dialog(msg, _("Error"), "dialog-error-symbolic");
 			break;	// avoid 100 error popups
 		}
 
@@ -644,7 +671,7 @@ static gpointer convert_thread_worker(gpointer p) {
 						goto clean_exit;
 					}
 				} else {
-					g_snprintf(dest_filename, 128, "%s%05d", destroot, indice++);
+					g_snprintf(dest_filename, 128, "%s%05d", args->destroot, indice++);
 					if (save_to_target_fits(fit, dest_filename)) {
 						siril_log_message(_("Error while converting to FITS (no space left?)\n"));
 						clearfits(fit);
@@ -675,7 +702,7 @@ static gpointer convert_thread_worker(gpointer p) {
 						break;
 					}
 				} else {
-					g_snprintf(dest_filename, 128, "%s%05d", destroot, indice++);
+					g_snprintf(dest_filename, 128, "%s%05d", args->destroot, indice++);
 					if (save_to_target_fits(fit, dest_filename)) {
 						siril_log_message(_("Error while converting to FITS (no space left?)\n"));
 						break;
@@ -699,92 +726,23 @@ clean_exit:
 			ser_write_and_close(ser_file);
 		free(ser_file);
 	}
-
-	gdk_threads_add_idle(end_convert_idle, args);
+	if (args->command_line) {
+		unset_debayer_in_convflags();
+	}
+	g_list_free_full(args->list, g_free);
+	g_dir_close(args->dir);
+	siril_add_idle(end_convert_idle, args);
 	return NULL;
 }
 
-static gboolean end_convert_idle(gpointer p) {
-	struct _convert_data *args = (struct _convert_data *) p;
-	struct timeval t_end;
-	
-	if (get_thread_run() && args->nb_converted > 1) {
-		// load the sequence
-		char *ppseqname = malloc(strlen(destroot) + 5);
-		sprintf(ppseqname, "%s.seq", destroot);
-		check_seq(0);
-		update_sequences_list(ppseqname);
-		free(ppseqname);
-	}
-	update_used_memory();
-	set_progress_bar_data(PROGRESS_TEXT_RESET, PROGRESS_DONE);
-	set_cursor_waiting(FALSE);
-	gettimeofday(&t_end, NULL);
-	show_time(args->t_start, t_end);
-	stop_processing_thread();
-	g_list_free_full(args->list, g_free);
-	g_dir_close(args->dir);
-	free(args);
-	return FALSE;
-}
-
-/* from a fits object, save to file or files, based on the channel policy from convflags */
-int save_to_target_fits(fits *fit, const char *dest_filename) {
-	if (convflags & CONV3X1) {	// an RGB image to 3 fits, one for each channel
-		char filename[130];
-
-		if (fit->naxis != 3) {
-			siril_log_message(_("Saving to 3 FITS files cannot be done because the source image does not have three channels\n"));
-			return 1;
-		}
-		sprintf(filename, "r_%s", dest_filename);
-		if (save1fits16(filename, fit, RLAYER)) {
-			printf("tofits: save1fit8 error, CONV3X1\n");
-			return 1;
-		}
-		sprintf(filename, "g_%s", dest_filename);
-		if (save1fits16(filename, fit, GLAYER)) {
-			printf("tofits: save1fit8 error, CONV3X1\n");
-			return 1;
-		}
-		sprintf(filename, "b_%s", dest_filename);
-		if (save1fits16(filename, fit, BLAYER)) {
-			printf("tofits: save1fit8 error, CONV3X1\n");
-			return 1;
-		}
-	} else if (convflags & CONV1X1) { // a single FITS to convert from an RGB grey image
-		if (save1fits16(dest_filename, fit, RLAYER)) {
-			printf("tofits: save1fit8 error, CONV1X1\n");
-			return 1;
-		}
-	} else {			// normal FITS save, any format
-		if (savefits(dest_filename, fit)) {
-			printf("tofits: savefit error, CONV1X3\n");
-			return 1;
-		}
-	}
-	return 0;
-}
-
-static int retrieveBayerPattern(char *bayer) {
-	int i;
-
-	for (i = 0; i < (sizeof(filter_pattern) / sizeof(char *)); i++) {
-		if (g_ascii_strcasecmp(bayer, filter_pattern[i]) == 0) {
-			return i;
-		}
-	}
-	return BAYER_FILTER_NONE;
-}
-
-int debayer_if_needed(image_type imagetype, fits *fit, gboolean compatibility) {
+int debayer_if_needed(image_type imagetype, fits *fit, gboolean compatibility, gboolean force_debayer) {
 	int retval = 0;
 	sensor_pattern tmp;
 	/* What the hell?
 	 * Siril's FITS are stored bottom to top, debayering will throw 
 	 * wrong results. So before demosacaing we need to transforme the image
 	 * with fits_flip_top_to_bottom() function */
-	if (imagetype == TYPEFITS && (convflags & CONVDEBAYER)) {
+	if (imagetype == TYPEFITS && (((convflags & CONVDEBAYER) && !force_debayer) || force_debayer)) {
 		tmp = com.debayer.bayer_pattern;
 		if (fit->naxes[2] != 1) {
 			siril_log_message(_("Cannot perform debayering on image with more than one channel\n"));
@@ -832,25 +790,6 @@ int debayer_if_needed(image_type imagetype, fits *fit, gboolean compatibility) {
 	return retval;
 }
 
-/* open the file with path source from any image type and load it into a new FITS object */
-fits *any_to_new_fits(image_type imagetype, const char *source, gboolean compatibility) {
-	int retval = 0;
-	fits *tmpfit = calloc(1, sizeof(fits));
-
-	retval = any_to_fits(imagetype, source, tmpfit);
-
-	if (!retval)
-		retval = debayer_if_needed(imagetype, tmpfit, compatibility);
-
-	if (retval) {
-		clearfits(tmpfit);
-		free(tmpfit);
-		return NULL;
-	}
-
-	return tmpfit;
-}
-
 /* open the file with path source from any image type and load it into the given FITS object */
 int any_to_fits(image_type imagetype, const char *source, fits *dest) {
 	int retval = 0;
@@ -902,3 +841,76 @@ int any_to_fits(image_type imagetype, const char *source, fits *dest) {
 	return retval;
 }
 
+void set_debayer_in_convflags() {
+	convflags |= CONVDEBAYER;
+}
+
+void unset_debayer_in_convflags() {
+	convflags &= ~CONVDEBAYER;
+}
+
+/******************Callback functions*******************************************************************/
+
+// truncates destroot if it's more than 120 characters, append a '_' if it
+// doesn't end with one or a '-'. SER extensions are accepted and unmodified.
+void on_convtoroot_changed (GtkEditable *editable, gpointer user_data){
+	static GtkWidget *multiple_ser = NULL;
+	const gchar *name = gtk_entry_get_text(GTK_ENTRY(editable));
+
+	if (*name != 0) {
+		if (!multiple_ser)
+			multiple_ser = lookup_widget("multipleSER");
+		if (destroot)
+			g_free(destroot);
+
+		destroot = g_str_to_ascii(name, NULL); // we want to avoid special char
+
+		const char *ext = get_filename_ext(destroot);
+		if (ext && !g_ascii_strcasecmp(ext, "ser")) {
+			convflags |= CONVDSTSER;
+			gtk_widget_set_visible(multiple_ser, TRUE);
+			return;
+		}
+		gtk_widget_set_visible(multiple_ser, FALSE);
+
+		destroot = format_basename(destroot);
+
+		check_for_conversion_form_completeness();
+	}
+}
+
+void on_demosaicing_toggled (GtkToggleButton *togglebutton, gpointer user_data) {
+	static GtkToggleButton *but = NULL;
+	if (!but) but = GTK_TOGGLE_BUTTON(lookup_widget("radiobutton1"));
+	if (gtk_toggle_button_get_active(togglebutton)) {
+		set_debayer_in_convflags();
+		gtk_toggle_button_set_active(but, TRUE);
+		com.debayer.open_debayer = TRUE;
+	}
+	else {
+		unset_debayer_in_convflags();	// used for conversion
+		com.debayer.open_debayer = FALSE;	// used for image opening
+	}
+}
+
+void on_multipleSER_toggled (GtkToggleButton *togglebutton, gpointer user_data) {
+	if (gtk_toggle_button_get_active(togglebutton))
+		convflags |= CONVMULTIPLE;
+	else convflags &= ~CONVMULTIPLE;
+}
+
+void on_conv3planefit_toggled (GtkToggleButton *togglebutton, gpointer user_data){
+	convflags |= CONV1X3;
+	convflags &= ~(CONV3X1|CONV1X1);
+}
+
+void on_conv3_1plane_toggled (GtkToggleButton *togglebutton, gpointer user_data) {
+	convflags |= CONV3X1;
+	convflags &= ~(CONV1X1|CONV1X3);
+}
+
+void on_conv1_1plane_toggled (GtkToggleButton *togglebutton, gpointer user_data) {
+	convflags |= CONV1X1;
+	convflags &= ~(CONV3X1|CONV1X3);
+}
+
