@@ -70,7 +70,7 @@ static int readtifstrip(TIFF* tif, uint32 width, uint32 height, uint16 nsamples,
 	WORD *gbuf[3] = {*data, *data, *data};
 	if (nsamples == 4) {
 		msg = siril_log_message(_("Alpha channel is ignored.\n"));
-		show_dialog(msg, _("Warning"), "dialog-warning");
+		show_dialog(msg, _("Warning"), "dialog-warning-symbolic");
 	}
 	if ((nsamples == 3) || (nsamples == 4)) {
 		gbuf[1] = *data + npixels;
@@ -86,7 +86,7 @@ static int readtifstrip(TIFF* tif, uint32 width, uint32 height, uint16 nsamples,
 			case PLANARCONFIG_CONTIG:
 				if (TIFFReadEncodedStrip(tif, TIFFComputeStrip(tif, row, 0), buf, nrow*scanline) < 0){
 					msg = siril_log_message(_("An unexpected error was encountered while trying to read the file.\n"));
-					show_dialog(msg, _("Error"), "dialog-error");
+					show_dialog(msg, _("Error"), "dialog-error-symbolic");
 					retval = -1;
 					break;
 				}
@@ -104,7 +104,7 @@ static int readtifstrip(TIFF* tif, uint32 width, uint32 height, uint16 nsamples,
 				for (j=0; j<nsamples; j++){	//loop on the layer
 					if (TIFFReadEncodedStrip(tif, TIFFComputeStrip(tif, row, j), buf, nrow*scanline) < 0){
 						msg = siril_log_message(_("An unexpected error was encountered while trying to read the file.\n"));
-						show_dialog(msg, _("Error"), "dialog-error");
+						show_dialog(msg, _("Error"), "dialog-error-symbolic");
 						retval = -1;
 						break;
 					}
@@ -114,7 +114,7 @@ static int readtifstrip(TIFF* tif, uint32 width, uint32 height, uint16 nsamples,
 				break;
 			default:
 				msg = siril_log_message(_("Unknown TIFF file.\n"));
-				show_dialog(msg, _("Error"), "dialog-error");
+				show_dialog(msg, _("Error"), "dialog-error-symbolic");
 				retval = -1;
 		}
 	}
@@ -156,7 +156,7 @@ static int readtif8bits(TIFF* tif, uint32 width, uint32 height, uint16 nsamples,
 		}
 		else {
 			msg = siril_log_message(_("An unexpected error was encountered while trying to read the file.\n"));
-			show_dialog(msg, _("Error"), "dialog-error");
+			show_dialog(msg, _("Error"), "dialog-error-symbolic");
 			retval = -1;
 		}
 		_TIFFfree(raster);
@@ -226,7 +226,7 @@ int readtif(const char *name, fits *fit) {
 
 		default :
 			msg = siril_log_message(_("Siril only works with 8/16-bit TIFF format.\n"));
-			show_dialog(msg, _("Warning"), "dialog-warning");
+			show_dialog(msg, _("Warning"), "dialog-warning-symbolic");
 			retval = -1;
 	}
 	TIFFClose(tif);
@@ -255,6 +255,7 @@ int readtif(const char *name, fits *fit) {
 			fit->pdata[BLAYER]=fit->data + npixels * 2;
 		}
 		fit->bitpix = (nbits == 8) ? BYTE_IMG : USHORT_IMG;
+		fit->orig_bitpix = fit->bitpix;
 		retval = nsamples;
 	}
 	if (nbits==16) mirrorx(fit, FALSE);
@@ -276,6 +277,7 @@ int savetif(const char *name, fits *fit, uint16 bitspersample){
 	WORD *buf16;
 	uint32 width, height, row, col, n;
 	uint16 nsamples;
+	double norm;
 
 	mirrorx(fit, FALSE);
 
@@ -288,7 +290,7 @@ int savetif(const char *name, fits *fit, uint16 bitspersample){
 
 	if (tif == NULL) {
 		msg = siril_log_message(_("Siril cannot create TIFF file.\n"));
-		show_dialog(msg, _("Error"), "dialog-error");
+		show_dialog(msg, _("Error"), "dialog-error-symbolic");
 		free(filename);
 		return 1;
 	}
@@ -345,7 +347,7 @@ int savetif(const char *name, fits *fit, uint16 bitspersample){
 	else {
 		TIFFClose(tif);
 		msg = siril_log_message(_("TIFF file has unexpected number of channels (not 1 or 3).\n"));
-		show_dialog(msg, _("Error"), "dialog-error");
+		show_dialog(msg, _("Error"), "dialog-error-symbolic");
 		free(filename);
 		return 1;
 	}
@@ -356,13 +358,13 @@ int savetif(const char *name, fits *fit, uint16 bitspersample){
 	switch (bitspersample) {
 	case 8:
 		buf8 = _TIFFmalloc(width * sizeof(unsigned char) * nsamples);
+		norm = fit->orig_bitpix > BYTE_IMG ? UCHAR_MAX_DOUBLE / USHRT_MAX_DOUBLE : 1.0;
 		for (row = 0; row < height; row++) {
 			for (col = 0; col < width; col++) {
 				for (n = 0; n < nsamples; n++) {
 					/* UCHAR_MAX / USHRT_MAX is constant, it's 1/255
 					 * This operation should be speed-up by doing a shift */
-					buf8[col * nsamples + n] = (*gbuf[n]++) * UCHAR_MAX_DOUBLE
-							/ USHRT_MAX_DOUBLE;
+					buf8[col * nsamples + n] = (*gbuf[n]++) * norm;
 				}
 			}
 			TIFFWriteScanline(tif, buf8, row, 0);
@@ -371,10 +373,12 @@ int savetif(const char *name, fits *fit, uint16 bitspersample){
 		break;
 	case 16:
 		buf16 = _TIFFmalloc(width * sizeof(WORD) * nsamples);
+		norm = (fit->orig_bitpix > BYTE_IMG) ? 1.0 : USHRT_MAX_DOUBLE / UCHAR_MAX_DOUBLE;
+
 		for (row = 0; row < height; row++) {
 			for (col = 0; col < width; col++) {
 				for (n = 0; n < nsamples; n++)
-					buf16[col * nsamples + n] = (*gbuf[n]++);
+					buf16[col * nsamples + n] = (*gbuf[n]++) * norm;
 			}
 			TIFFWriteScanline(tif, buf16, row, 0);
 		}
@@ -409,7 +413,7 @@ int readjpg(const char* name, fits *fit){
 
 	if ((f = g_fopen(name, "rb")) == NULL){
 		char *msg = siril_log_message(_("Sorry but Siril cannot open the file: %s.\n"), name);
-		show_dialog(msg, _("Error"), "dialog-error");
+		show_dialog(msg, _("Error"), "dialog-error-symbolic");
 		return -1;
 	}
 	cinfo.err = jpeg_std_error(&jerr);
@@ -436,7 +440,7 @@ int readjpg(const char* name, fits *fit){
 	jpeg_destroy_decompress(&cinfo);
 	if (data != NULL){
 		clearfits(fit);	
-		fit->bitpix = BYTE_IMG;
+		fit->bitpix = fit->orig_bitpix = BYTE_IMG;
 		if (cinfo.output_components==1)
 			fit->naxis = 2;
 		else
@@ -450,7 +454,7 @@ int readjpg(const char* name, fits *fit){
 		fit->pdata[RLAYER]=fit->data;
 		fit->pdata[GLAYER]=fit->data + npixels;
 		fit->pdata[BLAYER]=fit->data + npixels * 2;
-		fit->binning_x=fit->binning_y=1;
+		fit->binning_x = fit->binning_y = 1;
 	}
 	mirrorx(fit, FALSE);
 	gchar *basename = g_path_get_basename(name);
@@ -464,11 +468,12 @@ int readjpg(const char* name, fits *fit){
 int savejpg(const char *name, fits *fit, int quality){
 	FILE *f;
 	int i, j;
-	unsigned char red, blue, green;
+	WORD red, blue, green;
 	struct jpeg_compress_struct cinfo;    // Basic info for JPEG properties.
 	struct jpeg_error_mgr jerr;           // In case of error.
 	JSAMPROW row_pointer[1];              // Pointer to JSAMPLE row[s].
 	int row_stride;                       // Physical row width in image buffer.
+	double norm;
 
 	//## ALLOCATE AND INITIALIZE JPEG COMPRESSION OBJECT
 	cinfo.err = jpeg_std_error(&jerr);
@@ -482,7 +487,7 @@ int savejpg(const char *name, fits *fit, int quality){
 	//## OPEN FILE FOR DATA DESTINATION:
 	if ((f = g_fopen(filename, "wb")) == NULL) {
 		char *msg = siril_log_message(_("Siril cannot create JPG file.\n"));
-		show_dialog(msg, _("Error"), "dialog-error");
+		show_dialog(msg, _("Error"), "dialog-error-symbolic");
 		free(filename);
 		return 1;
 	}
@@ -491,48 +496,39 @@ int savejpg(const char *name, fits *fit, int quality){
 	//## SET PARAMETERS FOR COMPRESSION:
 	cinfo.image_width  = fit->rx;   // |-- Image width and height in pixels.
 	cinfo.image_height = fit->ry;   // |
-	cinfo.input_components = 3;     // Number of color components per pixel.
-	cinfo.in_color_space = JCS_RGB; // Colorspace of input image as RGB.
+	cinfo.input_components = fit->naxes[2];     // Number of color components per pixel.
+	cinfo.in_color_space = (fit->naxes[2] == 3) ? JCS_RGB : JCS_GRAYSCALE; // Colorspace of input image as RGB.
+
+	WORD *gbuf[3] =	{ fit->pdata[RLAYER], fit->pdata[GLAYER], fit->pdata[BLAYER] };
 
-	unsigned char *gbuf[3]={
-		com.graybuf[RLAYER]+cinfo.image_width*(cinfo.image_height-1)*4,
-		com.graybuf[GLAYER]+cinfo.image_width*(cinfo.image_height-1)*4,
-		com.graybuf[BLAYER]+cinfo.image_width*(cinfo.image_height-1)*4 };
 	jpeg_set_defaults(&cinfo);
 	jpeg_set_quality(&cinfo, quality, TRUE);
 
 	//## CREATE IMAGE BUFFER TO WRITE FROM AND MODIFY THE IMAGE TO LOOK LIKE CHECKERBOARD:
-	unsigned char *image_buffer = (unsigned char*)malloc(cinfo.image_width*cinfo.image_height*cinfo.num_components);
-	for (i=(cinfo.image_height-1);i>=0;i--){
-		for (j=0;j<cinfo.image_width;j++){
-			int pixelIdx = ((i*cinfo.image_width)+j) * cinfo.input_components;
-			red = *gbuf[RLAYER];
-			gbuf[RLAYER] += 4;
-			if (fit->naxes[2]==3){
-				green   = *gbuf[GLAYER];
-				blue    = *gbuf[BLAYER];
-				gbuf[GLAYER] += 4;
-				gbuf[BLAYER] += 4;
-			} else {
-				green   = red;
-				blue    = red;
+	unsigned char *image_buffer = (unsigned char*) malloc(
+			cinfo.image_width * cinfo.image_height * cinfo.num_components);
+
+	norm = (fit->orig_bitpix > BYTE_IMG ?
+			UCHAR_MAX_DOUBLE / USHRT_MAX_DOUBLE : 1.0);
+
+	for (i = (cinfo.image_height - 1); i >= 0; i--) {
+		for (j = 0; j < cinfo.image_width; j++) {
+			int pixelIdx = ((i * cinfo.image_width) + j) * cinfo.input_components;
+			red = *gbuf[RLAYER]++;
+			image_buffer[pixelIdx + 0] = round_to_BYTE(red * norm);         // r |-- Set r,g,b components to
+			if (cinfo.input_components == 3) {
+				green = *gbuf[GLAYER]++;
+				blue = *gbuf[BLAYER]++;
+				image_buffer[pixelIdx + 1] = round_to_BYTE(green * norm); // g |   make this pixel
+				image_buffer[pixelIdx + 2] = round_to_BYTE(blue * norm);  // b |
 			}
-			image_buffer[pixelIdx+0] = red;         // r |-- Set r,g,b components to
-			image_buffer[pixelIdx+1] = green;       // g |   make this pixel 
-			image_buffer[pixelIdx+2] = blue;        // b |
 		}
-		if (fit->naxes[2]==3){
-			gbuf[GLAYER] -= cinfo.image_width * 8;
-			gbuf[BLAYER] -= cinfo.image_width * 8;
-		}
-		gbuf[RLAYER] -= cinfo.image_width * 8;
 	}
 	//## START COMPRESSION:
 	jpeg_start_compress(&cinfo, TRUE);
-	row_stride = cinfo.image_width * 3;        // JSAMPLEs per row in image_buffer
+	row_stride = cinfo.image_width * cinfo.input_components;        // JSAMPLEs per row in image_buffer
 
-	while (cinfo.next_scanline < cinfo.image_height)
-	{
+	while (cinfo.next_scanline < cinfo.image_height) {
 		row_pointer[0] = &image_buffer[cinfo.next_scanline * row_stride];
 		(void) jpeg_write_scanlines(&cinfo, row_pointer, 1);
 	}
@@ -570,7 +566,7 @@ int readpng(const char *name, fits* fit) {
 			NULL);
 	if (!png) {
 		char *msg = siril_log_message(_("Sorry but Siril cannot open the file: %s.\n"), name);
-		show_dialog(msg, _("Error"), "dialog-error");
+		show_dialog(msg, _("Error"), "dialog-error-symbolic");
 		return -1;
 	}
 
@@ -643,7 +639,7 @@ int readpng(const char *name, fits* fit) {
 				*buf[BLAYER]++ = ptr[2];
 			}
 		}
-		fit->bitpix = BYTE_IMG;
+		fit->bitpix = fit->orig_bitpix = BYTE_IMG;
 	}
 	// We define the number of channel we have
 	switch (color_type) {
@@ -676,6 +672,7 @@ int readpng(const char *name, fits* fit) {
 		else
 			fit->naxis = 3;
 		fit->bitpix = (bit_depth == 8) ? BYTE_IMG : USHORT_IMG;
+		fit->orig_bitpix = fit->bitpix;
 		fit->data = data;
 		fit->pdata[RLAYER] = fit->data;
 		fit->pdata[GLAYER] = fit->data + npixels;
@@ -698,9 +695,10 @@ int readpng(const char *name, fits* fit) {
 // Save PNG image (colour)
 // ------------------------------------------
 static int32_t save_colour_file(const char *filename,
-		const WORD *p_image_data, uint32_t width, uint32_t height,
+		const void *p_image_data, uint32_t width, uint32_t height,
 		uint32_t bytes_per_sample) {
 	int32_t ret = -1;
+	int row_stride;
 	png_image image; // The control structure used by libpng
 
 	// Initialize the 'png_image' structure.
@@ -708,17 +706,19 @@ static int32_t save_colour_file(const char *filename,
 	image.version = PNG_IMAGE_VERSION;
 	image.width = width;
 	image.height = height;
-	image.format = PNG_FORMAT_BGR;
+	image.format = PNG_FORMAT_RGB;
 	if (bytes_per_sample == 2) {
 		image.format |= PNG_FORMAT_FLAG_LINEAR;
 	}
 
 	FILE *p_png_file = g_fopen(filename, "wb");
+	row_stride = image.width * -3;
 
 	if (p_png_file != NULL) {
-		png_image_write_to_stdio(&image, p_png_file, 0,  // convert_to_8bit
-				(png_bytep) (p_image_data), image.width * -3,  // row_stride
+		png_image_write_to_stdio(&image, p_png_file, 0,
+				(png_bytep) (p_image_data), row_stride,  // row_stride
 				NULL);  // colormap
+
 		ret = 0;
 		fclose(p_png_file);
 	}
@@ -729,9 +729,10 @@ static int32_t save_colour_file(const char *filename,
 // ------------------------------------------
 // Save PNG image (mono B, G or R)
 // ------------------------------------------
-static int32_t save_mono_file(const char *filename, const WORD *p_image_data,
+static int32_t save_mono_file(const char *filename, const void *p_image_data,
 		uint32_t width, uint32_t height, uint32_t bytes_per_sample) {
 	int32_t ret = -1;
+	int row_stride;
 	png_image image; // The control structure used by libpng
 
 	// Initialize the 'png_image' structure.
@@ -747,10 +748,11 @@ static int32_t save_mono_file(const char *filename, const WORD *p_image_data,
 	}
 
 	FILE *p_png_file = g_fopen(filename, "wb");
+	row_stride = image.width * -1;
 
 	if (p_png_file != NULL) {
 		png_image_write_to_stdio(&image, p_png_file, 0,  // convert_to_8bit
-				(png_bytep) p_image_data, image.width * -1,  // row_stride
+				(png_bytep) p_image_data, row_stride,  // row_stride
 				NULL);  // colormap
 		ret = 0;
 		fclose(p_png_file);
@@ -759,17 +761,36 @@ static int32_t save_mono_file(const char *filename, const WORD *p_image_data,
 	return ret;
 }
 
-static WORD *fits_to_bgrbgr(fits *image) {
+static WORD *convert_data(fits *image) {
 	int ndata = image->rx * image->ry;
+	int ch = image->naxes[2];
 	int i, j;
 
-	WORD *bgrbgr = malloc(ndata * 3 * sizeof(WORD));
-	for (i = 0, j = 0; i < ndata * 3; i += 3, j++) {
-		bgrbgr[i + 0] = image->pdata[BLAYER][j];
-		bgrbgr[i + 1] = image->pdata[GLAYER][j];
-		bgrbgr[i + 2] = image->pdata[RLAYER][j];
+	WORD *buffer = malloc(ndata * ch * sizeof(WORD));
+	for (i = 0, j = 0; i < ndata * ch; i += ch, j++) {
+		buffer[i + 0] = image->pdata[RLAYER][j];
+		if (ch > 1) {
+			buffer[i + 1] = image->pdata[GLAYER][j];
+			buffer[i + 2] = image->pdata[BLAYER][j];
+		}
 	}
-	return bgrbgr;
+	return buffer;
+}
+
+static uint8_t *convert_data8(fits *image) {
+	int ndata = image->rx * image->ry;
+	int ch = image->naxes[2];
+	int i, j;
+
+	uint8_t *buffer = malloc(ndata * ch * sizeof(WORD));
+	for (i = 0, j = 0; i < ndata * ch; i += ch, j++) {
+		buffer[i + 0] = (uint8_t)image->pdata[RLAYER][j];
+		if (ch > 1) {
+			buffer[i + 1] = (uint8_t) image->pdata[GLAYER][j];
+			buffer[i + 2] = (uint8_t) image->pdata[BLAYER][j];
+		}
+	}
+	return buffer;
 }
 
 int savepng(const char *name, fits *fit, uint32_t bytes_per_sample,
@@ -785,14 +806,28 @@ int savepng(const char *name, fits *fit, uint32_t bytes_per_sample,
 
 	if (is_colour) {
 		// Create colour PNG file
-		WORD *data = fits_to_bgrbgr(fit);
-		ret = save_colour_file(filename, data, width, height,
-				bytes_per_sample);
-		free(data);
+		if (bytes_per_sample == 2) {
+			WORD *data = convert_data(fit);
+			ret = save_colour_file(filename, data, width, height,
+					bytes_per_sample);
+			free(data);
+		} else {
+			uint8_t *data = convert_data8(fit);
+			ret = save_colour_file(filename, data, width, height,
+					bytes_per_sample);
+			free(data);
+		}
 	} else {
 		// Create monochrome PNG file
-		ret = save_mono_file(filename, fit->data, width, height,
-				bytes_per_sample);
+		if (bytes_per_sample == 2) {
+			ret = save_mono_file(filename, fit->data, width, height,
+					bytes_per_sample);
+		} else {
+			uint8_t *data = convert_data8(fit);
+			ret = save_mono_file(filename, data, width, height,
+					bytes_per_sample);
+			free(data);
+		}
 	}
 	siril_log_message(_("Saving PNG: file %s, %ld layer(s), %ux%u pixels\n"),
 			filename, fit->naxes[2], fit->rx, fit->ry);
@@ -1019,7 +1054,7 @@ static int readraw(const char *name, fits *fit) {
 	
 	if (data != NULL) {
 		clearfits(fit);
-		fit->bitpix = USHORT_IMG;
+		fit->bitpix = fit->orig_bitpix = USHORT_IMG;
 		fit->rx = (unsigned int) width;
 		fit->ry = (unsigned int) height;
 		fit->naxes[0] = (long) width;
@@ -1187,7 +1222,7 @@ static int readraw_in_cfa(const char *name, fits *fit) {
 	
 	if (data != NULL) {
 		clearfits(fit);
-		fit->bitpix = USHORT_IMG;
+		fit->bitpix = fit->orig_bitpix = USHORT_IMG;
 		fit->rx = (unsigned int) (width);
 		fit->ry = (unsigned int) (height);
 		fit->naxes[0] = (long) (width);
