@@ -1,5 +1,6 @@
 #include <string.h>
 #include "core/siril.h"
+#include "algos/statistics.h"
 #include "stacking.h"
 #include "io/sequence.h"
 #include "core/proto.h"
@@ -37,14 +38,19 @@ static int _compute_normalization_for_image(struct stacking_args *args, int i, i
 		double *offset, double *mul, double *scale, normalization mode, double *scale0,
 		double *mul0, double *offset0) {
 	imstats *stat = NULL;
+	int reglayer;
 
-	if (!(stat = seq_get_imstats(args->seq, args->image_indices[i], NULL, STATS_EXTRA))) {
-		fits fit;
-		memset(&fit, 0, sizeof(fits));
+	reglayer = (args->reglayer == -1) ? 0 : args->reglayer;
+
+	// try with no fit passed: fails if data is needed because data is not cached
+	if (!(stat = statistics(args->seq, args->image_indices[i], NULL, reglayer, NULL, STATS_EXTRA))) {
+		fits fit = { 0 };
 		if (seq_read_frame(args->seq, args->image_indices[i], &fit)) {
 			return 1;
 		}
-		stat = seq_get_imstats(args->seq, args->image_indices[i], &fit, STATS_EXTRA);
+		// retry with the fit to compute it
+		if (!(stat = statistics(args->seq, args->image_indices[i], &fit, reglayer, NULL, STATS_EXTRA)))
+			return 1;
 		if (args->seq->type != SEQ_INTERNAL)
 			clearfits(&fit);
 	}
@@ -55,7 +61,7 @@ static int _compute_normalization_for_image(struct stacking_args *args, int i, i
 		scale[i] = stat->scale;
 		if (i == ref_image)
 			*scale0 = scale[ref_image];
-		scale[i] = *scale0 / scale[i];
+		scale[i] = (scale[i] == 0) ? 1 : *scale0 / scale[i];
 		/* no break */
 	case ADDITIVE:
 		offset[i] = stat->location;
@@ -67,15 +73,17 @@ static int _compute_normalization_for_image(struct stacking_args *args, int i, i
 		scale[i] = stat->scale;
 		if (i == ref_image)
 			*scale0 = scale[ref_image];
-		scale[i] = *scale0 / scale[i];
+		scale[i] = (scale[i] == 0) ? 1 : *scale0 / scale[i];
 		/* no break */
 	case MULTIPLICATIVE:
 		mul[i] = stat->location;
 		if (i == ref_image)
 			*mul0 = mul[ref_image];
-		mul[i] = *mul0 / mul[i];
+		mul[i] = (mul[i] == 0) ? 1 : *mul0 / mul[i];
 		break;
 	}
+
+	free_stats(stat);
 	return 0;
 }
 
@@ -107,22 +115,15 @@ static int compute_normalization(struct stacking_args *args) {
 			break;
 		}
 	if (ref_image_filtred_idx == -1) {
-		char *msg = siril_log_color_message(_("The reference image is not in the selected set of images. "
+		siril_log_color_message(_("The reference image is not in the selected set of images. "
 				"Please choose another reference image.\n"), "red");
-		show_dialog(msg, _("Error"), "dialog-error");
 		siril_log_color_message(_("Normalisation skipped.\n"), "red");
 		return 1;
 	}
 
 	/* We empty the cache if needed (force to recompute) */
-	if (args->force_norm) {
-		for (i = 0; i < args->seq->number; i++) {
-			if (args->seq->imgparam && args->seq->imgparam[i].stats) {
-				free(args->seq->imgparam[i].stats);
-				args->seq->imgparam[i].stats = NULL;
-			}
-		}
-	}
+	if (args->force_norm)
+		clear_stats(args->seq, args->reglayer);
 
 	// compute for the first image to have scale0 mul0 and offset0
 	if (_compute_normalization_for_image(args,
